@@ -25,18 +25,12 @@
 #include <config.h>
 #include "eel-background.h"
 #include "eel-gdk-extensions.h"
-#include "eel-gdk-pixbuf-extensions.h"
 #include "eel-glib-extensions.h"
-#include "eel-gnome-extensions.h"
-#include "eel-gtk-macros.h"
 #include "eel-lib-self-check-functions.h"
-#include "eel-string.h"
-#include "eel-marshal.h"
-#include "eel-types.h"
-#include "eel-type-builtins.h"
 #include <gtk/gtk.h>
 #include <eel/eel-canvas.h>
 #include <eel/eel-canvas-util.h>
+#include <cairo-xlib.h>
 #include <gdk/gdkx.h>
 #include <gio/gio.h>
 #include <math.h>
@@ -44,20 +38,16 @@
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 #include <libgnomeui/gnome-bg.h>
 
-static void       eel_background_class_init                (gpointer       klass);
-static void       eel_background_init                      (gpointer       object,
-							    gpointer       klass);
-static void       eel_background_finalize                  (GObject       *object);
-static GdkPixmap *eel_background_get_pixmap_and_color      (EelBackground *background,
-							    GdkWindow     *window,
-							    GdkColor      *color);
+static cairo_surface_t *eel_background_get_surface_and_color  (EelBackground *background,
+							      GdkWindow     *window,
+							      GdkColor      *color);
 
 static void set_image_properties (EelBackground *background);
 
 static void init_fade (EelBackground *background, GtkWidget *widget);
 static void free_fade (EelBackground *background);
 
-EEL_CLASS_BOILERPLATE (EelBackground, eel_background, GTK_TYPE_OBJECT)
+G_DEFINE_TYPE (EelBackground, eel_background, G_TYPE_OBJECT);
 
 enum {
 	APPEARANCE_CHANGED,
@@ -80,8 +70,8 @@ struct EelBackgroundDetails {
 	GtkWidget *widget;
 
 	/* Realized data: */
-	GdkPixmap *background_pixmap;
-	gboolean background_pixmap_is_unset_root_pixmap;
+	cairo_surface_t *background_surface;
+	gboolean background_surface_is_unset_root_surface;
 	GnomeBGCrossfade *fade;
 	int background_entire_width;
 	int background_entire_height;
@@ -95,57 +85,14 @@ struct EelBackgroundDetails {
 	gulong screen_size_handler;
 	/* Desktop monitors configuration watcher */
 	gulong screen_monitors_handler;
-	/* Can we use common pixmap for root window and desktop window */
-	gboolean use_common_pixmap;
+	/* Can we use common surface for root window and desktop window */
+	gboolean use_common_surface;
 	guint change_idle_id;
 
 	/* activity status */
 	gboolean is_active;
 };
 
-static void
-eel_background_class_init (gpointer klass)
-{
-	GObjectClass *object_class;
-
-	object_class = G_OBJECT_CLASS (klass);
-
-	eel_type_init ();
-
-	signals[APPEARANCE_CHANGED] =
-		g_signal_new ("appearance_changed",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE,
-			      G_STRUCT_OFFSET (EelBackgroundClass,
-					       appearance_changed),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__VOID,
-			      G_TYPE_NONE,
-			      0);
-	signals[SETTINGS_CHANGED] =
-		g_signal_new ("settings_changed",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE,
-			      G_STRUCT_OFFSET (EelBackgroundClass,
-					       settings_changed),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__INT,
-			      G_TYPE_NONE,
-			      1, G_TYPE_INT);
-	signals[RESET] =
-		g_signal_new ("reset",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE,
-			      G_STRUCT_OFFSET (EelBackgroundClass,
-					       reset),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__VOID,
-			      G_TYPE_NONE,
-			      0);
-
-	object_class->finalize = eel_background_finalize;
-}
-
 static void
 on_bg_changed (GnomeBG *bg, EelBackground *background)
 {
@@ -163,13 +110,13 @@ on_bg_transitioned (GnomeBG *bg, EelBackground *background)
 }
 
 static void
-eel_background_init (gpointer object, gpointer klass)
+eel_background_init (EelBackground *background)
 {
-	EelBackground *background;
+	background->details =
+		G_TYPE_INSTANCE_GET_PRIVATE (background,
+					     EEL_TYPE_BACKGROUND,
+					     EelBackgroundDetails);
 
-	background = EEL_BACKGROUND (object);
-
-	background->details = g_new0 (EelBackgroundDetails, 1);
 	background->details->default_color.red = 0xffff;
 	background->details->default_color.green = 0xffff;
 	background->details->default_color.blue = 0xffff;
@@ -207,47 +154,26 @@ free_fade (EelBackground *background)
 }
 
 static void
-free_background_pixmap (EelBackground *background)
+free_background_surface (EelBackground *background)
 {
-	GdkDisplay *display;
-	GdkPixmap *pixmap;
+	cairo_surface_t *surface;
 
-	pixmap = background->details->background_pixmap;
-	if (pixmap != NULL) {
-		/* If we created a root pixmap and didn't set it as background
+	surface = background->details->background_surface;
+	if (surface != NULL) {
+		/* If we created a root surface and didn't set it as background
 		   it will live forever, so we need to kill it manually.
 		   If set as root background it will be killed next time the
 		   background is changed. */
-		if (background->details->background_pixmap_is_unset_root_pixmap) {
-			display = gdk_drawable_get_display (GDK_DRAWABLE (pixmap));
-			XKillClient (GDK_DISPLAY_XDISPLAY (display),
-				     GDK_PIXMAP_XID (pixmap));
+		if (background->details->background_surface_is_unset_root_surface) {
+			XKillClient (cairo_xlib_surface_get_display (surface),
+				     cairo_xlib_surface_get_drawable (surface));
 		}
-		g_object_unref (pixmap);
-		background->details->background_pixmap = NULL;
+		cairo_surface_destroy (surface);
+		background->details->background_surface = NULL;
 	}
 }
 
 
-static void
-eel_background_finalize (GObject *object)
-{
-	EelBackground *background;
-
-	background = EEL_BACKGROUND (object);
-
-	g_free (background->details->color);
-	eel_background_remove_current_image (background);
-
-	free_background_pixmap (background);
-
-	free_fade (background);
-
-	g_free (background->details);
-
-	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
-}
-
 static EelBackgroundImagePlacement
 placement_gnome_to_eel (GnomeBGPlacement p)
 {
@@ -308,6 +234,67 @@ eel_background_set_image_placement (EelBackground              *background,
 				placement_eel_to_gnome (new_placement));
 }
 
+
+static void
+eel_background_finalize (GObject *object)
+{
+	EelBackground *background;
+
+	background = EEL_BACKGROUND (object);
+
+	g_free (background->details->color);
+	eel_background_remove_current_image (background);
+
+	free_background_surface (background);
+
+	free_fade (background);
+
+	G_OBJECT_CLASS (eel_background_parent_class)->finalize (object);
+}
+
+static void
+eel_background_class_init (EelBackgroundClass *klass)
+{
+	GObjectClass *object_class;
+
+	object_class = G_OBJECT_CLASS (klass);
+
+	signals[APPEARANCE_CHANGED] =
+		g_signal_new ("appearance_changed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE,
+			      G_STRUCT_OFFSET (EelBackgroundClass,
+					       appearance_changed),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
+	signals[SETTINGS_CHANGED] =
+		g_signal_new ("settings_changed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE,
+			      G_STRUCT_OFFSET (EelBackgroundClass,
+					       settings_changed),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__INT,
+			      G_TYPE_NONE,
+			      1, G_TYPE_INT);
+	signals[RESET] =
+		g_signal_new ("reset",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE,
+			      G_STRUCT_OFFSET (EelBackgroundClass,
+					       reset),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
+
+	object_class->finalize = eel_background_finalize;
+
+	g_type_class_add_private (klass, sizeof (EelBackgroundDetails));
+}
+
 EelBackground *
 eel_background_new (void)
 {
@@ -317,7 +304,7 @@ eel_background_new (void)
 static void
 eel_background_unrealize (EelBackground *background)
 {
-	free_background_pixmap (background);
+	free_background_surface (background);
 	
 	background->details->background_entire_width = 0;
 	background->details->background_entire_height = 0;
@@ -328,19 +315,20 @@ eel_background_unrealize (EelBackground *background)
 
 static void
 drawable_get_adjusted_size (EelBackground *background,
-			    GdkDrawable   *drawable,
+			    GdkWindow     *window,
 			    int		  *width,
 			    int	          *height)
 {
 	GdkScreen *screen;
-	
+
 	/* 
 	 * Screen resolution change makes root drawable have incorrect size.
-	 */    
-	gdk_drawable_get_size (drawable, width, height);
+	 */
+	*width = gdk_window_get_width (window);
+	*height = gdk_window_get_height (window);
 
 	if (background->details->is_desktop) {
-		screen = gdk_drawable_get_screen (drawable);
+		screen = gdk_window_get_screen (window);
 		*width = gdk_screen_get_width (screen);
 		*height = gdk_screen_get_height (screen);
 	}
@@ -370,8 +358,6 @@ eel_background_ensure_realized (EelBackground *background, GdkWindow *window)
 		} else {
 			background->details->default_color = style->bg[GTK_STATE_NORMAL];
 		}
-		
-		gdk_rgb_find_color (style->colormap, &(background->details->default_color));
 	}
 
 	/* If the window size is the same as last time, don't update */
@@ -380,19 +366,19 @@ eel_background_ensure_realized (EelBackground *background, GdkWindow *window)
 		return FALSE;
 	}
 
-	free_background_pixmap (background);
+	free_background_surface (background);
 
 	changed = FALSE;
 
 	set_image_properties (background);
 
-	background->details->background_pixmap = gnome_bg_create_pixmap (background->details->bg,
-									 window,
-									 entire_width, entire_height,
-									 background->details->is_desktop);
-	background->details->background_pixmap_is_unset_root_pixmap = background->details->is_desktop;
+	background->details->background_surface = gnome_bg_create_surface (background->details->bg,
+									   window,
+									   entire_width, entire_height,
+									   background->details->is_desktop);
+	background->details->background_surface_is_unset_root_surface = background->details->is_desktop;
 		
-	/* We got the pixmap and everything, so we don't care about a change
+	/* We got the surface and everything, so we don't care about a change
 	   that is pending (unless things actually change after this time) */
 	g_object_set_data (G_OBJECT (background->details->bg),
 			   "ignore-pending-change", GINT_TO_POINTER (TRUE));
@@ -437,10 +423,10 @@ make_color_inactive (EelBackground *background, GdkColor *color)
 	}
 }
 
-static GdkPixmap *
-eel_background_get_pixmap_and_color (EelBackground *background,
-				     GdkWindow     *window,
-				     GdkColor      *color)
+static cairo_surface_t *
+eel_background_get_surface_and_color (EelBackground *background,
+				      GdkWindow     *window,
+				      GdkColor      *color)
 {
 	int entire_width;
 	int entire_height;
@@ -452,66 +438,12 @@ eel_background_get_pixmap_and_color (EelBackground *background,
 	*color = background->details->default_color;
 	make_color_inactive (background, color);
 
-	if (background->details->background_pixmap != NULL) {
-		return g_object_ref (background->details->background_pixmap);
+	if (background->details->background_surface != NULL) {
+		return cairo_surface_reference (background->details->background_surface);
 	} 
 	return NULL;
 }
 
-void
-eel_background_expose (GtkWidget                   *widget,
-		       GdkEventExpose              *event)
-{
-	GdkColor color;
-	int window_width;
-	int window_height;
-	GdkPixmap *pixmap;
-	GdkGC *gc;
-	GdkGCValues gc_values;
-	GdkGCValuesMask value_mask;
-	GdkWindow *widget_window;
-
-	EelBackground *background;
-
-	widget_window = gtk_widget_get_window (widget);
-	if (event->window != widget_window) {
-		return;
-	}
-
-	background = eel_get_widget_background (widget);
-
-	drawable_get_adjusted_size (background, widget_window, &window_width, &window_height);
-	
-	pixmap = eel_background_get_pixmap_and_color (background,
-						      widget_window,
-						      &color);
-
-	if (pixmap) {
-		gc_values.tile = pixmap;
-		gc_values.ts_x_origin = 0;
-		gc_values.ts_y_origin = 0;
-		gc_values.fill = GDK_TILED;
-		value_mask = GDK_GC_FILL | GDK_GC_TILE | GDK_GC_TS_X_ORIGIN | GDK_GC_TS_Y_ORIGIN;
-	} else {
-		gdk_rgb_find_color (gtk_widget_get_colormap (widget), &color);
-		gc_values.foreground = color;
-		gc_values.fill = GDK_SOLID;
-		value_mask = GDK_GC_FILL | GDK_GC_FOREGROUND;
-	}
-	
-	gc = gdk_gc_new_with_values (widget_window, &gc_values, value_mask);
-	
-	gdk_gc_set_clip_rectangle (gc, &event->area);
-
-	gdk_draw_rectangle (widget_window, gc, TRUE, 0, 0, window_width, window_height);
-	
-	g_object_unref (gc);
-	
-	if (pixmap) {
-		g_object_unref (pixmap);
-	}
-}
-
 static void
 set_image_properties (EelBackground *background)
 {
@@ -582,7 +514,7 @@ void
 eel_background_set_color (EelBackground *background,
 			  const char *color)
 {
-	if (eel_strcmp (background->details->color, color) != 0) {
+	if (g_strcmp0 (background->details->color, color) != 0) {
 		g_free (background->details->color);
 		background->details->color = g_strdup (color);
 		
@@ -607,7 +539,7 @@ eel_background_set_image_uri_helper (EelBackground *background,
 	gnome_bg_set_filename (background->details->bg, filename);
 
 	if (emit_signal) {
-		g_signal_emit (GTK_OBJECT (background), signals[SETTINGS_CHANGED], 0, GDK_ACTION_COPY);
+		g_signal_emit (background, signals[SETTINGS_CHANGED], 0, GDK_ACTION_COPY);
 	}
 
 	set_image_properties (background);
@@ -682,56 +614,65 @@ eel_background_reset (EelBackground *background)
 {
 	g_return_if_fail (EEL_IS_BACKGROUND (background));
 
-	g_signal_emit (GTK_OBJECT (background), signals[RESET], 0);
+	g_signal_emit (background, signals[RESET], 0);
 }
 
 static void
-set_root_pixmap (EelBackground *background,
-                 GdkWindow     *window)
+set_root_surface (EelBackground *background,
+		  GdkWindow     *window)
 {
-	GdkPixmap *pixmap, *root_pixmap;
+	cairo_surface_t *surface, *root_surface;
 	GdkScreen *screen;
 	GdkColor color;
 
-	pixmap = eel_background_get_pixmap_and_color (background,
-						      window,
-						      &color);
-	screen = gdk_drawable_get_screen (window);
+	surface = eel_background_get_surface_and_color (background,
+							window,
+							&color);
+	screen = gdk_window_get_screen (window);
 
-	if (background->details->use_common_pixmap) {
-		background->details->background_pixmap_is_unset_root_pixmap = FALSE;
-		root_pixmap = g_object_ref (pixmap);
+	if (background->details->use_common_surface) {
+		background->details->background_surface_is_unset_root_surface = FALSE;
+		root_surface = cairo_surface_reference (surface);
 	} else {
-		root_pixmap = gnome_bg_create_pixmap (background->details->bg, window,
-						      gdk_screen_get_width (screen), gdk_screen_get_height (screen), TRUE);
+		root_surface = gnome_bg_create_surface (background->details->bg, window,
+							gdk_screen_get_width (screen), gdk_screen_get_height (screen), TRUE);
 	}
 
-	gnome_bg_set_pixmap_as_root (screen, pixmap);
+	gnome_bg_set_surface_as_root (screen, surface);
 
-	g_object_unref (pixmap);
-	g_object_unref (root_pixmap);
+	cairo_surface_destroy (surface);
+	cairo_surface_destroy (root_surface);
+}
+
+
+static void
+on_fade_finished (GnomeBGCrossfade *fade,
+		  GdkWindow *window,
+		  EelBackground *background)
+{
+	set_root_surface (background, window);
 }
 
 static gboolean
-fade_to_pixmap (EelBackground *background,
+fade_to_surface (EelBackground *background,
 		 GdkWindow     *window,
-		 GdkPixmap     *pixmap)
+		 cairo_surface_t *surface)
 {
 	if (background->details->fade == NULL) {
 		return FALSE;
 	}
 
-	if (!gnome_bg_crossfade_set_end_pixmap (background->details->fade,
-				                pixmap)) {
+	if (!gnome_bg_crossfade_set_end_surface (background->details->fade,
+				                 surface)) {
 		return FALSE;
 	}
 
 	if (!gnome_bg_crossfade_is_started (background->details->fade)) {
 		gnome_bg_crossfade_start (background->details->fade, window);
 		if (background->details->is_desktop) {
-			g_signal_connect_swapped (background->details->fade,
-					          "finished",
-						  G_CALLBACK (set_root_pixmap), background);
+			g_signal_connect (background->details->fade,
+					  "finished",
+					  G_CALLBACK (on_fade_finished), background);
 		}
 	}
 
@@ -743,7 +684,7 @@ static void
 eel_background_set_up_widget (EelBackground *background, GtkWidget *widget)
 {
 	GtkStyle *style;
-	GdkPixmap *pixmap;
+	cairo_surface_t *surface;
 	GdkColor color;
 	
 	int window_width;
@@ -760,14 +701,12 @@ eel_background_set_up_widget (EelBackground *background, GtkWidget *widget)
 	widget_window = gtk_widget_get_window (widget);
 	drawable_get_adjusted_size (background, widget_window, &window_width, &window_height);
 	
-	pixmap = eel_background_get_pixmap_and_color (background,
-						      widget_window,
-						      &color);
+	surface = eel_background_get_surface_and_color (background,
+						       widget_window,
+						       &color);
 
 	style = gtk_widget_get_style (widget);
 	
-	gdk_rgb_find_color (style->colormap, &color);
-
 	if (EEL_IS_CANVAS (widget)) {
 		window = gtk_layout_get_bin_window (GTK_LAYOUT (widget));
 	} else {
@@ -775,26 +714,30 @@ eel_background_set_up_widget (EelBackground *background, GtkWidget *widget)
 	}
 
 	if (background->details->fade != NULL) {
-		in_fade = fade_to_pixmap (background, window, pixmap);
+		in_fade = fade_to_surface (background, window, surface);
 	} else {
 		in_fade = FALSE;
 	}
 
 	if (!in_fade) {
-		if (background->details->is_desktop) {
-			gdk_window_set_back_pixmap (window, pixmap, FALSE);
-		} else {
+		cairo_pattern_t *pattern;
+
+		pattern = cairo_pattern_create_for_surface (surface);
+		gdk_window_set_background_pattern (window, pattern);
+
+		if (!background->details->is_desktop) {
 			gdk_window_set_background (window, &color);
-			gdk_window_set_back_pixmap (window, pixmap, FALSE);
 		}
+
+		cairo_pattern_destroy (pattern);
 	}
 
 	if (background->details->is_desktop && !in_fade) {
-		set_root_pixmap (background, window);
+		set_root_surface (background, window);
 	}
 	
-	if (pixmap) {
-		g_object_unref (pixmap);
+	if (surface) {
+		cairo_surface_destroy (surface);
 	}
 }
 
@@ -833,7 +776,8 @@ init_fade (EelBackground *background, GtkWidget *widget)
 		 * we don't want to crossfade
 		 */
 		window = gtk_widget_get_window (widget);
-		gdk_drawable_get_size (window, &old_width, &old_height);
+		old_width = gdk_window_get_width (window);
+		old_height = gdk_window_get_height (window);
 		drawable_get_adjusted_size (background, window,
 					    &width, &height);
 		if (old_width == width && old_height == height) {
@@ -846,16 +790,16 @@ init_fade (EelBackground *background, GtkWidget *widget)
 	}
 
 	if (background->details->fade != NULL && !gnome_bg_crossfade_is_started (background->details->fade)) {
-		GdkPixmap *start_pixmap;
+		cairo_surface_t *start_surface;
 
-		if (background->details->background_pixmap == NULL) {
-			start_pixmap = gnome_bg_get_pixmap_from_root (gtk_widget_get_screen (widget));
+		if (background->details->background_surface == NULL) {
+			start_surface = gnome_bg_get_surface_from_root (gtk_widget_get_screen (widget));
 		} else {
-			start_pixmap = g_object_ref (background->details->background_pixmap);
+			start_surface = cairo_surface_reference (background->details->background_surface);
 		}
-		gnome_bg_crossfade_set_start_pixmap (background->details->fade,
-						     start_pixmap);
-                g_object_unref (start_pixmap);
+		gnome_bg_crossfade_set_start_surface (background->details->fade,
+						      start_surface);
+                cairo_surface_destroy (start_surface);
 	}
 }
 
@@ -925,10 +869,10 @@ widget_realized_setup (GtkWidget *widget, gpointer data)
 
 		root_window = gdk_screen_get_root_window(screen);			
 		
-		if (gdk_drawable_get_visual (root_window) == gtk_widget_get_visual (widget)) {
-			background->details->use_common_pixmap = TRUE;
+		if (gdk_window_get_visual (root_window) == gtk_widget_get_visual (widget)) {
+			background->details->use_common_surface = TRUE;
 		} else {
-			background->details->use_common_pixmap = FALSE;
+			background->details->use_common_surface = FALSE;
 		}
 
 		init_fade (background, widget);
@@ -964,7 +908,7 @@ widget_unrealize_cb (GtkWidget *widget, gpointer data)
 				                     background->details->screen_monitors_handler);
 			background->details->screen_monitors_handler = 0;
 	}
-	background->details->use_common_pixmap = FALSE;
+	background->details->use_common_surface = FALSE;
 }
 
 void
@@ -1026,7 +970,6 @@ eel_get_widget_background (GtkWidget *widget)
 
 	/* Store the background in the widget's data. */
 	background = eel_background_new ();
-	g_object_ref_sink (background);
 	g_object_set_data_full (G_OBJECT (widget), "eel_background",
 				background, g_object_unref);
 	background->details->widget = widget;
@@ -1066,66 +1009,7 @@ eel_background_is_dark (EelBackground *background)
 	gdk_screen_get_monitor_geometry (screen, 0, &rect);
 
 	return gnome_bg_is_dark (background->details->bg, rect.width, rect.height);
-}
-   
-/* handle dropped colors */
-void
-eel_background_receive_dropped_color (EelBackground *background,
-				      GtkWidget *widget,
-				      GdkDragAction action,
-				      int drop_location_x,
-				      int drop_location_y,
-				      const GtkSelectionData *selection_data)
-{
-	guint16 *channels;
-	char *color_spec;
-	char *new_gradient_spec;
-	int left_border, right_border, top_border, bottom_border;
-	GtkAllocation allocation;
-
-	g_return_if_fail (EEL_IS_BACKGROUND (background));
-	g_return_if_fail (GTK_IS_WIDGET (widget));
-	g_return_if_fail (selection_data != NULL);
-
-	/* Convert the selection data into a color spec. */
-	if (gtk_selection_data_get_length ((GtkSelectionData *) selection_data) != 8 ||
-	    gtk_selection_data_get_format ((GtkSelectionData *) selection_data) != 16) {
-		g_warning ("received invalid color data");
-		return;
-	}
-	channels = (guint16 *) gtk_selection_data_get_data ((GtkSelectionData *) selection_data);
-	color_spec = g_strdup_printf ("#%02X%02X%02X",
-				      channels[0] >> 8,
-				      channels[1] >> 8,
-				      channels[2] >> 8);
-
-	/* Figure out if the color was dropped close enough to an edge to create a gradient.
-	   For the moment, this is hard-wired, but later the widget will have to have some
-	   say in where the borders are.
-	*/
-	gtk_widget_get_allocation (widget, &allocation);
-	left_border = 32;
-	right_border = allocation.width - 32;
-	top_border = 32;
-	bottom_border = allocation.height - 32;
-	if (drop_location_x < left_border && drop_location_x <= right_border) {
-		new_gradient_spec = eel_gradient_set_left_color_spec (background->details->color, color_spec);
-	} else if (drop_location_x >= left_border && drop_location_x > right_border) {
-		new_gradient_spec = eel_gradient_set_right_color_spec (background->details->color, color_spec);
-	} else if (drop_location_y < top_border && drop_location_y <= bottom_border) {
-		new_gradient_spec = eel_gradient_set_top_color_spec (background->details->color, color_spec);
-	} else if (drop_location_y >= top_border && drop_location_y > bottom_border) {
-		new_gradient_spec = eel_gradient_set_bottom_color_spec (background->details->color, color_spec);
-	} else {
-		new_gradient_spec = g_strdup (color_spec);
-	}
-	
-	g_free (color_spec);
-
-	eel_background_set_image_uri_and_color (background, action, NULL, new_gradient_spec);
-
-	g_free (new_gradient_spec);
-}
+}   
 
 void
 eel_background_save_to_gconf (EelBackground *background)
