@@ -70,10 +70,6 @@ static const GtkTargetEntry drop_types [] = {
 	/* prefer "_NETSCAPE_URL" over "text/uri-list" to satisfy web browsers. */
 	{ NAUTILUS_ICON_DND_NETSCAPE_URL_TYPE, 0, NAUTILUS_ICON_DND_NETSCAPE_URL },
 	{ NAUTILUS_ICON_DND_URI_LIST_TYPE, 0, NAUTILUS_ICON_DND_URI_LIST },
-	{ NAUTILUS_ICON_DND_COLOR_TYPE, 0, NAUTILUS_ICON_DND_COLOR },
-	{ NAUTILUS_ICON_DND_BGIMAGE_TYPE, 0, NAUTILUS_ICON_DND_BGIMAGE },
-	{ NAUTILUS_ICON_DND_KEYWORD_TYPE, 0, NAUTILUS_ICON_DND_KEYWORD },
-	{ NAUTILUS_ICON_DND_RESET_BACKGROUND_TYPE,  0, NAUTILUS_ICON_DND_RESET_BACKGROUND },
 	{ NAUTILUS_ICON_DND_XDNDDIRECTSAVE_TYPE, 0, NAUTILUS_ICON_DND_XDNDDIRECTSAVE }, /* XDS Protocol Type */
 	{ NAUTILUS_ICON_DND_RAW_TYPE, 0, NAUTILUS_ICON_DND_RAW },
 	/* Must be last: */
@@ -96,7 +92,6 @@ create_selection_shadow (NautilusIconContainer *container,
 {
 	EelCanvasGroup *group;
 	EelCanvas *canvas;
-	GdkBitmap *stipple;
 	int max_x, max_y;
 	int min_x, min_y;
 	GList *p;
@@ -111,9 +106,6 @@ create_selection_shadow (NautilusIconContainer *container,
 		return NULL;
 	}
 		
-	stipple = container->details->dnd_info->stipple;
-	g_return_val_if_fail (stipple != NULL, NULL);
-
 	canvas = EEL_CANVAS (container);
 	gtk_widget_get_allocation (GTK_WIDGET (container), &allocation);
 
@@ -158,7 +150,7 @@ create_selection_shadow (NautilusIconContainer *container,
 				 "x2", (double) x2,
 				 "y2", (double) y2,
 				 "outline_color", "black",
-				 "outline_stipple", stipple,
+				 "outline-stippling", TRUE,
 				 "width_pixels", 1,
 				 NULL);
 	}
@@ -366,7 +358,7 @@ nautilus_icon_container_dropped_icon_feedback (GtkWidget *widget,
 	if (dnd_info->shadow != NULL) {
 		/* FIXME bugzilla.gnome.org 42484: 
 		 * Is a destroy really sufficient here? Who does the unref? */
-		gtk_object_destroy (GTK_OBJECT (dnd_info->shadow));
+		eel_canvas_item_destroy (dnd_info->shadow);
 	}
 
 	/* Build the selection list and the shadow. */
@@ -605,114 +597,6 @@ nautilus_icon_container_selection_items_local (NautilusIconContainer *container,
 	return result;
 }
 
-static GdkDragAction 
-get_background_drag_action (NautilusIconContainer *container, 
-			    GdkDragAction action)
-{
-	/* FIXME: This function is very FMDirectoryView specific, and
-	 * should be moved out of nautilus-icon-dnd.c */
-	GdkDragAction valid_actions;
-
-	if (action == GDK_ACTION_ASK) {
-		valid_actions = NAUTILUS_DND_ACTION_SET_AS_FOLDER_BACKGROUND;
-		if (!eel_background_is_desktop (eel_get_widget_background (GTK_WIDGET (container)))) {
-			valid_actions |= NAUTILUS_DND_ACTION_SET_AS_GLOBAL_BACKGROUND;
-		}
-
-		action = nautilus_drag_drop_background_ask 
-			(GTK_WIDGET (container), valid_actions);
-	}
-
-	return action;
-}
-
-static void
-receive_dropped_color (NautilusIconContainer *container,
-		       int x, int y,
-		       GdkDragAction action,
-		       GtkSelectionData *data)
-{
-	action = get_background_drag_action (container, action);
-
-	if (action > 0) {
-		char *uri;
-
-		uri = get_container_uri (container);
-		nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
-				    "dropped color on icon container displaying %s", uri);
-		g_free (uri);
-
-		eel_background_receive_dropped_color
-			(eel_get_widget_background (GTK_WIDGET (container)),
-			 GTK_WIDGET (container), 
-			 action, x, y, data);
-	}
-}
-
-/* handle dropped tile images */
-static void
-receive_dropped_tile_image (NautilusIconContainer *container, GdkDragAction action, GtkSelectionData *data)
-{
-	g_assert (data != NULL);
-
-	action = get_background_drag_action (container, action);
-
-	if (action > 0) {
-		char *uri;
-
-		uri = get_container_uri (container);
-		nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
-				    "dropped tile image on icon container displaying %s", uri);
-		g_free (uri);
-
-		eel_background_receive_dropped_background_image
-			(eel_get_widget_background (GTK_WIDGET (container)), 
-			 action, 
-			 gtk_selection_data_get_data (data));
-	}
-}
-
-/* handle dropped keywords */
-static void
-receive_dropped_keyword (NautilusIconContainer *container, const char *keyword, int x, int y)
-{
-	char *uri;
-	double world_x, world_y;
-
-	NautilusIcon *drop_target_icon;
-	NautilusFile *file;
-	
-	g_assert (keyword != NULL);
-
-	/* find the item we hit with our drop, if any */
-	canvas_widget_to_world (EEL_CANVAS (container), x, y, &world_x, &world_y);
-	drop_target_icon = nautilus_icon_container_item_at (container, world_x, world_y);
-	if (drop_target_icon == NULL) {
-		return;
-	}
-
-	/* FIXME bugzilla.gnome.org 42485: 
-	 * This does not belong in the icon code.
-	 * It has to be in the file manager.
-	 * The icon code has no right to deal with the file directly.
-	 * But luckily there's no issue of not getting a file object,
-	 * so we don't have to worry about async. issues here.
-	 */
-	uri = nautilus_icon_container_get_icon_uri (container, drop_target_icon);
-
-	nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
-			    "dropped emblem '%s' on icon container URI: %s",
-			    keyword, uri);
-
-	file = nautilus_file_get_by_uri (uri);
-	g_free (uri);
-	
-	nautilus_drag_file_receive_dropped_keyword (file, keyword);
-
-	nautilus_file_unref (file);
-	nautilus_icon_container_update_icon (container, drop_target_icon);
-}
-
 /* handle dropped url */
 static void
 receive_dropped_netscape_url (NautilusIconContainer *container, const char *encoded_url, GdkDragContext *context, int x, int y)
@@ -893,54 +777,6 @@ stop_auto_scroll (NautilusIconContainer *container)
 	nautilus_drag_autoscroll_stop (&container->details->dnd_info->drag_info);
 }
 
-static gboolean
-confirm_switch_to_manual_layout (NautilusIconContainer *container)
-{
-#if 0
-	const char *message;
-	const char *detail;
-	GtkDialog *dialog;
-	int response;
-
-	/* FIXME bugzilla.gnome.org 40915: Use of the word "directory"
-	 * makes this FMIconView specific. Move these messages into
-	 * FMIconView so NautilusIconContainer can be used for things
-	 * that are not directories?
-	 */
-	if (nautilus_icon_container_has_stored_icon_positions (container)) {
-		if (eel_g_list_exactly_one_item (container->details->dnd_info->drag_info.selection_list)) {
-			message = no_translate("Do you want to switch to manual layout and leave this item where you dropped it? "
-			"This will clobber the stored manual layout.");
-			detail = no_translate("This folder uses automatic layout.");
-		} else {
-			message = no_translate("Do you want to switch to manual layout and leave these items where you dropped them? "
-			"This will clobber the stored manual layout.");
-			detail = no_translate("This folder uses automatic layout.");
-		}
-	} else {
-		if (eel_g_list_exactly_one_item (container->details->dnd_info->drag_info.selection_list)) {
-			message = no_translate("Do you want to switch to manual layout and leave this item where you dropped it?");
-			detail = no_translate("This folder uses automatic layout.");
-		} else {
-			message = no_translate("Do you want to switch to manual layout and leave these items where you dropped them?");
-			detail = no_translate("This folder uses automatic layout.");
-
-		}
-	}
-
-	dialog = eel_show_yes_no_dialog (message, detail, _("Switch to Manual Layout?"),
-					 GTK_STOCK_CANCEL,
-					 GTK_WINDOW (gtk_widget_get_toplevel(GTK_WIDGET(container))));
-
-	response = gtk_dialog_run (dialog);
-	gtk_object_destroy (GTK_OBJECT (dialog));
-
-	return response == GTK_RESPONSE_YES;
-#else
-	return FALSE;
-#endif
-}
-
 static void
 handle_local_move (NautilusIconContainer *container,
 		   double world_x, double world_y)
@@ -954,10 +790,7 @@ handle_local_move (NautilusIconContainer *container,
 	time_t now;
 
 	if (container->details->auto_layout) {
-		if (!confirm_switch_to_manual_layout (container)) {
-			return;
-		}
-		nautilus_icon_container_freeze_icon_positions (container);
+		return;
 	}
 
 	time (&now);
@@ -1010,7 +843,7 @@ handle_local_move (NautilusIconContainer *container,
 
 static void
 handle_nonlocal_move (NautilusIconContainer *container,
-		      GdkDragContext *context,
+		      GdkDragAction action,
 		      int x, int y,
 		      const char *target_uri,
 		      gboolean icon_hit)
@@ -1067,11 +900,11 @@ handle_nonlocal_move (NautilusIconContainer *container,
 
 	/* start the copy */
 	g_signal_emit_by_name (container, "move_copy_items",
-				 source_uris,
-				 source_item_locations,
-				 target_uri,
-				 gdk_drag_context_get_selected_action (context),
-				 x, y);
+			       source_uris,
+			       source_item_locations,
+			       target_uri,
+			       action,
+			       x, y);
 
 	if (free_target_uri) {
 		g_free ((char *)target_uri);
@@ -1197,7 +1030,7 @@ nautilus_icon_container_receive_dropped_icons (NautilusIconContainer *container,
 					       GdkDragContext *context,
 					       int x, int y)
 {
-	char *drop_target;
+	char *drop_target, *container_uri;
 	gboolean local_move_only;
 	double world_x, world_y;
 	gboolean icon_hit;
@@ -1220,16 +1053,20 @@ nautilus_icon_container_receive_dropped_icons (NautilusIconContainer *container,
 			action = GDK_ACTION_MOVE;
 		} else {
 			action = GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK;
+			container_uri = get_container_uri (container);
 			
-			if (selection_is_image_file (container->details->dnd_info->drag_info.selection_list)) {
+			if (eel_uri_is_desktop (container_uri) &&
+			    selection_is_image_file (container->details->dnd_info->drag_info.selection_list)) {
 				action |= NAUTILUS_DND_ACTION_SET_AS_BACKGROUND;
 			}
+
+			g_free (container_uri);
 		}
 		real_action = nautilus_drag_drop_action_ask
 			(GTK_WIDGET (container), action);
 	}
 	
-	if (real_action == NAUTILUS_DND_ACTION_SET_AS_BACKGROUND) {
+	if (real_action == (GdkDragAction) NAUTILUS_DND_ACTION_SET_AS_BACKGROUND) {
 		selected_item = container->details->dnd_info->drag_info.selection_list->data;
 		eel_background_receive_dropped_background_image
 			(eel_get_widget_background (GTK_WIDGET (container)),
@@ -1259,7 +1096,7 @@ nautilus_icon_container_receive_dropped_icons (NautilusIconContainer *container,
 		if (local_move_only) {
 			handle_local_move (container, world_x, world_y);
 		} else {
-			handle_nonlocal_move (container, context, world_x, world_y, drop_target, icon_hit);
+			handle_nonlocal_move (container, real_action, world_x, world_y, drop_target, icon_hit);
 		}
 	}
 
@@ -1316,20 +1153,10 @@ nautilus_icon_container_get_drop_action (NautilusIconContainer *container,
 		g_free (drop_target);
 		break;
 
-	/* handle emblems by setting the action if we're over an object */
-	case NAUTILUS_ICON_DND_KEYWORD:
-		if (icon != NULL) {
-			*action = gdk_drag_context_get_suggested_action (context);
-		}
-		break;
-
 	case NAUTILUS_ICON_DND_NETSCAPE_URL:
 		*action = nautilus_drag_default_drop_action_for_netscape_url (context);
 		break;
 
-	case NAUTILUS_ICON_DND_COLOR:
-	case NAUTILUS_ICON_DND_BGIMAGE:
-	case NAUTILUS_ICON_DND_RESET_BACKGROUND:
 	case NAUTILUS_ICON_DND_ROOTWINDOW_DROP:
 		*action = gdk_drag_context_get_suggested_action (context);
 		break;
@@ -1386,7 +1213,7 @@ nautilus_icon_dnd_update_drop_target (NautilusIconContainer *container,
 	 */
 
 	/* Find if target icon accepts our drop. */
-	if (icon != NULL && (container->details->dnd_info->drag_info.data_type != NAUTILUS_ICON_DND_KEYWORD)) {
+	if (icon != NULL) {
 		    uri = nautilus_icon_container_get_icon_uri (container, icon);
 		    file = nautilus_file_get_by_uri (uri);
 		    g_free (uri);
@@ -1413,7 +1240,7 @@ nautilus_icon_container_free_drag_data (NautilusIconContainer *container)
 	dnd_info->drag_info.got_drop_data_type = FALSE;
 
 	if (dnd_info->shadow != NULL) {
-		gtk_object_destroy (GTK_OBJECT (dnd_info->shadow));
+		eel_canvas_item_destroy (dnd_info->shadow);
 		dnd_info->shadow = NULL;
 	}
 
@@ -1454,42 +1281,19 @@ drag_begin_callback (GtkWidget      *widget,
 		     gpointer        data)
 {
 	NautilusIconContainer *container;
-	GdkScreen *screen;
-	GdkColormap *colormap;
-	GdkPixmap *pixmap;
-	GdkBitmap *mask;
+	cairo_surface_t *surface;
 	double x1, y1, x2, y2, winx, winy;
 	int x_offset, y_offset;
 	int start_x, start_y;
-	gboolean use_mask;
 
 	container = NAUTILUS_ICON_CONTAINER (widget);
 
-	screen = gtk_widget_get_screen (widget);
-	colormap = NULL;
-	if (gdk_screen_is_composited (screen)) {
-		colormap = gdk_screen_get_rgba_colormap (screen);
-		if (colormap != NULL) {
-			use_mask = FALSE;
-		}
-	}
-	
-	/* Fall back on using the same colormap as the widget */
-	if (colormap == NULL) {
-		colormap = gtk_widget_get_colormap (widget);		
-		use_mask = TRUE;
-	}
-
 	start_x = container->details->dnd_info->drag_info.start_x + gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (container)));
 	start_y = container->details->dnd_info->drag_info.start_y + gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (container)));
 
         /* create a pixmap and mask to drag with */
-        pixmap = nautilus_icon_canvas_item_get_image (container->details->drag_icon->item, &mask, colormap);
-    
-    	/* we want to drag semi-transparent pixbufs, but X is too slow dealing with
-	   stippled masks, so we had to remove the code; this comment is left as a memorial
-	   to it, with the hope that we get it back someday as X Windows improves */
-	
+        surface = nautilus_icon_canvas_item_get_drag_surface (container->details->drag_icon->item);
+
         /* compute the image's offset */
 	eel_canvas_item_get_bounds (EEL_CANVAS_ITEM (container->details->drag_icon->item),
 				    &x1, &y1, &x2, &y2);
@@ -1498,21 +1302,9 @@ drag_begin_callback (GtkWidget      *widget,
         x_offset = start_x - winx;
         y_offset = start_y - winy;
 
-	if (!use_mask && pixmap != NULL) {
-		cairo_t *cr;
-
-		/* If composite works, make the icons partially transparent */
-		cr = gdk_cairo_create (pixmap);
-		cairo_set_operator (cr, CAIRO_OPERATOR_DEST_OUT);
-		cairo_set_source_rgba(cr, 1,0,0,0.35);
-		cairo_paint (cr);
-		cairo_destroy (cr);
-	}
-	
-	gtk_drag_set_icon_pixmap (context,
-				  colormap,
-				  pixmap, (use_mask ? mask : NULL),
-				  x_offset, y_offset);
+        cairo_surface_set_device_offset (surface, -x_offset, -y_offset);
+        gtk_drag_set_icon_surface (context, surface);
+        cairo_surface_destroy (surface);
 }
 
 void
@@ -1547,29 +1339,31 @@ nautilus_icon_dnd_begin_drag (NautilusIconContainer *container,
 }
 
 static gboolean
-drag_highlight_expose (GtkWidget      *widget,
-		       GdkEventExpose *event,
-		       gpointer        data)
+drag_highlight_draw (GtkWidget *widget,
+                     cairo_t   *cr,
+                     gpointer   user_data)
 {
 	gint x, y, width, height;
 	GdkWindow *window;
 	
 	x = gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (widget)));
 	y = gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (widget)));
-	gdk_drawable_get_size (gtk_widget_get_window (widget), &width, &height);
 
-	window = gtk_layout_get_bin_window (GTK_LAYOUT (widget));
-	
-	gtk_paint_shadow (gtk_widget_get_style (widget), window,
+        window = gtk_widget_get_window (widget);
+        width = gdk_window_get_width (window);
+        height = gdk_window_get_height (window);
+
+	gtk_paint_shadow (gtk_widget_get_style (widget),
+                          cr,
 			  GTK_STATE_NORMAL, GTK_SHADOW_OUT,
-			  NULL, widget, "dnd",
+			  widget, "dnd",
 			  x, y, width, height);
-  
-	gdk_draw_rectangle (window,
-			    (gtk_widget_get_style(widget))->black_gc,
-			    FALSE,
-			    x, y, width - 1, height - 1);
-	
+
+	cairo_set_line_width (cr, 1.0);
+	cairo_set_source_rgb (cr, 0, 0, 0);
+	cairo_rectangle (cr, x + 0.5, y + 0.5, width - 1, height - 1);
+	cairo_stroke (cr);
+
 	return FALSE;
 }
 
@@ -1624,8 +1418,8 @@ start_dnd_highlight (GtkWidget *widget)
 
 	if (!dnd_info->highlighted) {
 		dnd_info->highlighted = TRUE;
-		g_signal_connect_after (widget, "expose_event",
-					G_CALLBACK (drag_highlight_expose),
+		g_signal_connect_after (widget, "draw",
+					G_CALLBACK (drag_highlight_draw),
 					NULL);
 		dnd_highlight_queue_redraw (widget);
 	}
@@ -1640,7 +1434,7 @@ stop_dnd_highlight (GtkWidget *widget)
 
 	if (dnd_info->highlighted) {
 		g_signal_handlers_disconnect_by_func (widget,
-						      drag_highlight_expose,
+						      drag_highlight_draw,
 						      NULL);
 		dnd_highlight_queue_redraw (widget);
 		dnd_info->highlighted = FALSE;
@@ -1732,7 +1526,6 @@ drag_data_received_callback (GtkWidget *widget,
 			     gpointer user_data)
 {
     	NautilusDragInfo *drag_info;
-	EelBackground *background;
 	char *tmp;
 	const char *tmp_raw;
 	int length;
@@ -1747,12 +1540,8 @@ drag_data_received_callback (GtkWidget *widget,
 	case NAUTILUS_ICON_DND_GNOME_ICON_LIST:
 		nautilus_icon_container_dropped_icon_feedback (widget, data, x, y);
 		break;
-	case NAUTILUS_ICON_DND_COLOR:
-	case NAUTILUS_ICON_DND_BGIMAGE:	
-	case NAUTILUS_ICON_DND_KEYWORD:
 	case NAUTILUS_ICON_DND_URI_LIST:
 	case NAUTILUS_ICON_DND_TEXT:
-	case NAUTILUS_ICON_DND_RESET_BACKGROUND:
 	case NAUTILUS_ICON_DND_XDNDDIRECTSAVE:
 	case NAUTILUS_ICON_DND_RAW:
 		/* Save the data so we can do the actual work on drop. */
@@ -1786,24 +1575,6 @@ drag_data_received_callback (GtkWidget *widget,
 				(NAUTILUS_ICON_CONTAINER (widget),
 				 context, x, y);
 			break;
-		case NAUTILUS_ICON_DND_COLOR:
-			receive_dropped_color (NAUTILUS_ICON_CONTAINER (widget),
-					       x, y,
-					       gdk_drag_context_get_selected_action (context),
-					       data);
-			success = TRUE;
-			break;
-		case NAUTILUS_ICON_DND_BGIMAGE:
-			receive_dropped_tile_image
-				(NAUTILUS_ICON_CONTAINER (widget),
-				 gdk_drag_context_get_selected_action (context),
-				 data);
-			break;
-		case NAUTILUS_ICON_DND_KEYWORD:
-			receive_dropped_keyword
-				(NAUTILUS_ICON_CONTAINER (widget),
-				 (char *) gtk_selection_data_get_data (data), x, y);
-			break;
 		case NAUTILUS_ICON_DND_NETSCAPE_URL:
 			receive_dropped_netscape_url
 				(NAUTILUS_ICON_CONTAINER (widget),
@@ -1833,13 +1604,6 @@ drag_data_received_callback (GtkWidget *widget,
 				 context, x, y);
 			success = TRUE;
 			break;
-		case NAUTILUS_ICON_DND_RESET_BACKGROUND:
-			background = eel_get_widget_background (widget);
-			if (background != NULL) {
-				eel_background_reset (background);
-			}
-			gtk_drag_finish (context, FALSE, FALSE, time);			
-			break;
 		case NAUTILUS_ICON_DND_ROOTWINDOW_DROP:
 			/* Do nothing, everything is done by the sender */
 			break;
@@ -1897,23 +1661,7 @@ drag_data_received_callback (GtkWidget *widget,
 }
 
 void
-nautilus_icon_dnd_set_stipple (NautilusIconContainer *container,
-			       GdkBitmap             *stipple)
-{
-	if (stipple != NULL) {
-		g_object_ref (stipple);
-	}
-	
-	if (container->details->dnd_info->stipple != NULL) {
-		g_object_unref (container->details->dnd_info->stipple);
-	}
-
-	container->details->dnd_info->stipple = stipple;
-}
-
-void
-nautilus_icon_dnd_init (NautilusIconContainer *container,
-			GdkBitmap *stipple)
+nautilus_icon_dnd_init (NautilusIconContainer *container)
 {
 	GtkTargetList *targets;
 	int n_elements;
@@ -1961,10 +1709,6 @@ nautilus_icon_dnd_init (NautilusIconContainer *container,
 			  G_CALLBACK (drag_drop_callback), NULL);
 	g_signal_connect (container, "drag_leave",
 			  G_CALLBACK (drag_leave_callback), NULL);
-
-	if (stipple != NULL) {
-		container->details->dnd_info->stipple = g_object_ref (stipple);
-	}
 }
 
 void
@@ -1975,10 +1719,6 @@ nautilus_icon_dnd_fini (NautilusIconContainer *container)
 	if (container->details->dnd_info != NULL) {
 		stop_auto_scroll (container);
 
-		if (container->details->dnd_info->stipple != NULL) {
-			g_object_unref (container->details->dnd_info->stipple);
-		}
-
 		nautilus_drag_finalize (&container->details->dnd_info->drag_info);
 		container->details->dnd_info = NULL;
 	}
