@@ -198,7 +198,7 @@ G_DEFINE_TYPE_WITH_CODE (FMIconView, fm_icon_view, FM_TYPE_DIRECTORY_VIEW,
 						fm_icon_view_iface_init));
 
 static void
-fm_icon_view_destroy (GtkObject *object)
+fm_icon_view_destroy (GtkWidget *object)
 {
 	FMIconView *icon_view;
 
@@ -223,7 +223,7 @@ fm_icon_view_destroy (GtkObject *object)
 		icon_view->details->icons_not_positioned = NULL;
 	}
 
-	GTK_OBJECT_CLASS (fm_icon_view_parent_class)->destroy (object);
+	GTK_WIDGET_CLASS (fm_icon_view_parent_class)->destroy (object);
 }
 
 
@@ -1219,27 +1219,6 @@ fm_icon_view_begin_loading (FMDirectoryView *view)
 	/* kill any sound preview process that is ongoing */
 	preview_audio (icon_view, NULL, FALSE);
 
-	/* FIXME bugzilla.gnome.org 45060: Should use methods instead
-	 * of hardcoding desktop knowledge in here.
-	 */
-	if (FM_IS_DESKTOP_ICON_VIEW (view)) {
-		nautilus_connect_desktop_background_to_file_metadata (NAUTILUS_ICON_CONTAINER (icon_container), file);
-	} else {
-		GdkDragAction default_action;
-		
-		if (nautilus_window_info_get_window_type (fm_directory_view_get_nautilus_window (view)) == NAUTILUS_WINDOW_NAVIGATION) {
-			default_action = NAUTILUS_DND_ACTION_SET_AS_GLOBAL_BACKGROUND;
-		} else {
-			default_action = NAUTILUS_DND_ACTION_SET_AS_FOLDER_BACKGROUND;
-		}
-		
-		nautilus_connect_background_to_file_metadata 
-			(icon_container, 
-			 file, 
-			 default_action);
-	}
-
-	
 	/* Set up the zoom level from the metadata. */
 	if (fm_directory_view_supports_zooming (FM_DIRECTORY_VIEW (icon_view))) {
 		if (icon_view->details->compact) {
@@ -2650,8 +2629,6 @@ fm_icon_view_sort_directories_first_changed (FMDirectoryView *directory_view)
 	}
 }
 
-/* GtkObject methods. */
-
 static gboolean
 icon_view_can_accept_item (NautilusIconContainer *container,
 			   NautilusFile *target_item,
@@ -2954,8 +2931,7 @@ fm_icon_view_class_init (FMIconViewClass *klass)
 	G_OBJECT_CLASS (klass)->set_property = fm_icon_view_set_property;
 	G_OBJECT_CLASS (klass)->finalize = fm_icon_view_finalize;
 
-	GTK_OBJECT_CLASS (klass)->destroy = fm_icon_view_destroy;
-	
+	GTK_WIDGET_CLASS (klass)->destroy = fm_icon_view_destroy;
 	GTK_WIDGET_CLASS (klass)->screen_changed = fm_icon_view_screen_changed;
 	GTK_WIDGET_CLASS (klass)->scroll_event = fm_icon_view_scroll_event;
 	
