@@ -29,8 +29,8 @@
 
 #include <gtk/gtk.h>
 
-void     nautilus_main_event_loop_register    (GtkObject *object);
-gboolean nautilus_main_is_event_loop_mainstay (GtkObject *object);
+void     nautilus_main_event_loop_register    (GtkWidget *object);
+gboolean nautilus_main_is_event_loop_mainstay (GtkWidget *object);
 void     nautilus_main_event_loop_quit        (gboolean explicit);
 
 #endif /* NAUTILUS_MAIN_H */
