@@ -43,7 +43,6 @@
 #include "nautilus-navigation-window-pane.h"
 #include "nautilus-src-marshal.h"
 #include <eel/eel-debug.h>
-#include <eel/eel-marshal.h>
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-string.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
@@ -188,10 +187,10 @@ nautilus_window_init (NautilusWindow *window)
 	g_signal_connect_object (nautilus_signaller_get_current (), "popup_menu_changed",
 			 G_CALLBACK (nautilus_window_load_extension_menus), window, G_CONNECT_SWAPPED);
 
-	gtk_quit_add_destroy (1, GTK_OBJECT (window));
+	gtk_quit_add_destroy (1, GTK_WIDGET (window));
 
 	/* Keep the main event loop alive as long as the window exists */
-	nautilus_main_event_loop_register (GTK_OBJECT (window));
+	nautilus_main_event_loop_register (GTK_WIDGET (window));
 }
 
 /* Unconditionally synchronize the GtkUIManager of WINDOW. */
@@ -234,6 +233,17 @@ nautilus_window_go_to (NautilusWindow *window, GFile *location)
 	nautilus_window_slot_go_to (window->details->active_pane->active_slot, location, FALSE);
 }
 
+void
+nautilus_window_go_to_full (NautilusWindow *window,
+			    GFile          *location,
+			    NautilusWindowGoToCallback callback,
+			    gpointer        user_data)
+{
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
+
+	nautilus_window_slot_go_to_full (window->details->active_pane->active_slot, location, FALSE, callback, user_data);
+}
+
 void
 nautilus_window_go_to_with_selection (NautilusWindow *window, GFile *location, GList *new_selection)
 {
@@ -319,7 +329,8 @@ nautilus_window_go_up (NautilusWindow *window, gboolean close_behind, gboolean n
 	nautilus_window_slot_open_location_full (slot, parent, 
 						 NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
 						 flags,
-						 selection);
+						 selection,
+						 NULL, NULL);
 	
 	g_object_unref (parent);
 	
@@ -597,7 +608,7 @@ free_stored_viewers (NautilusWindow *window)
 }
 
 static void
-nautilus_window_destroy (GtkObject *object)
+nautilus_window_destroy (GtkWidget *object)
 {
 	NautilusWindow *window;
 	GList *panes_copy;
@@ -613,7 +624,7 @@ nautilus_window_destroy (GtkObject *object)
 	g_assert (window->details->panes == NULL);
 	g_assert (window->details->active_pane == NULL);
 
-	GTK_OBJECT_CLASS (nautilus_window_parent_class)->destroy (object);
+	GTK_WIDGET_CLASS (nautilus_window_parent_class)->destroy (object);
 }
 
 static void
@@ -1085,7 +1096,7 @@ add_view_as_menu_item (NautilusWindow *window,
 		g_snprintf (accel_path, sizeof (accel_path), "<Nautilus-Window>/%s", action_name);
 
 		accel_keyval = gdk_keyval_from_name (accel);
-		g_assert (accel_keyval != GDK_VoidSymbol);
+		g_assert (accel_keyval != GDK_KEY_VoidSymbol);
 
 		gtk_accel_map_add_entry (accel_path, accel_keyval, GDK_CONTROL_MASK);
 		gtk_action_set_accel_path (GTK_ACTION (action), accel_path);
@@ -1986,7 +1997,7 @@ nautilus_window_class_init (NautilusWindowClass *class)
 	G_OBJECT_CLASS (class)->constructed = nautilus_window_constructed;
 	G_OBJECT_CLASS (class)->get_property = nautilus_window_get_property;
 	G_OBJECT_CLASS (class)->set_property = nautilus_window_set_property;
-	GTK_OBJECT_CLASS (class)->destroy = nautilus_window_destroy;
+	GTK_WIDGET_CLASS (class)->destroy = nautilus_window_destroy;
 	GTK_WIDGET_CLASS (class)->show = nautilus_window_show;
 	GTK_WIDGET_CLASS (class)->size_request = nautilus_window_size_request;
 	GTK_WIDGET_CLASS (class)->realize = nautilus_window_realize;
@@ -2048,12 +2059,12 @@ nautilus_window_class_init (NautilusWindowClass *class)
 			      G_TYPE_NONE, 0);
 
 	binding_set = gtk_binding_set_by_class (class);
-	gtk_binding_entry_add_signal (binding_set, GDK_BackSpace, 0,
+	gtk_binding_entry_add_signal (binding_set, GDK_KEY_BackSpace, 0,
 				      "go_up", 1,
 				      G_TYPE_BOOLEAN, FALSE);
-	gtk_binding_entry_add_signal (binding_set, GDK_F5, 0,
+	gtk_binding_entry_add_signal (binding_set, GDK_KEY_F5, 0,
 				      "reload", 0);
-	gtk_binding_entry_add_signal (binding_set, GDK_slash, 0,
+	gtk_binding_entry_add_signal (binding_set, GDK_KEY_slash, 0,
 				      "prompt-for-location", 1,
 				      G_TYPE_STRING, "/");
 
